@@ -137,7 +137,9 @@ create_list_widget (gpointer item,
   GtkWidget *checkmark = g_object_new (GTK_TYPE_IMAGE,
                                        "halign", GTK_ALIGN_START,
                                        "icon-name", "emblem-ok-symbolic",
+                                       "opacity", 0,
                                        "valign", GTK_ALIGN_CENTER,
+                                       "visible", TRUE,
                                        NULL);
   GtkWidget *box = g_object_new (GTK_TYPE_BOX,
                                  "child", priv->create_list_widget_func (item, priv->create_widget_func_data),
@@ -178,20 +180,22 @@ update (HdyComboRow *self)
   g_autoptr(GObject) item = NULL;
   g_autofree gchar *name = NULL;
   GtkWidget *widget;
+  guint n_items = priv->bound_model ? g_list_model_get_n_items (priv->bound_model) : 0;
 
   gtk_widget_set_visible (GTK_WIDGET (priv->current), !priv->use_subtitle);
   gtk_container_foreach (GTK_CONTAINER (priv->current), (GtkCallback) gtk_widget_destroy, NULL);
 
-  if (priv->bound_model == NULL || g_list_model_get_n_items (priv->bound_model) == 0) {
-    gtk_widget_set_sensitive (GTK_WIDGET (self), FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (self), n_items > 0);
+  gtk_widget_set_visible (GTK_WIDGET (priv->image), n_items > 1);
+  gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (self), n_items > 1);
+
+  if (n_items == 0) {
     g_assert (priv->selected_index == -1);
 
     return;
   }
 
-  g_assert (priv->selected_index >= 0 && priv->selected_index <= g_list_model_get_n_items (priv->bound_model));
-
-  gtk_widget_set_sensitive (GTK_WIDGET (self), TRUE);
+  g_assert (priv->selected_index >= 0 && priv->selected_index <= n_items);
 
   {
     g_autoptr (GList) rows = gtk_container_get_children (GTK_CONTAINER (priv->list));
@@ -202,8 +206,8 @@ update (HdyComboRow *self)
       GtkWidget *row = GTK_WIDGET (l->data);
       GtkWidget *box = gtk_bin_get_child (GTK_BIN (row));
 
-      gtk_widget_set_visible (GTK_WIDGET (g_object_get_data (G_OBJECT (box), "checkmark")),
-                              priv->selected_index == i++);
+      gtk_widget_set_opacity (GTK_WIDGET (g_object_get_data (G_OBJECT (box), "checkmark")),
+                              (priv->selected_index == i++) ? 1 : 0);
     }
   }
 
@@ -233,8 +237,10 @@ bound_model_changed (GListModel *list,
   HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
 
   /* Selection is in front of insertion/removal point, nothing to do */
-  if (priv->selected_index > 0 && priv->selected_index < index)
+  if (priv->selected_index > 0 && priv->selected_index < index) {
+    update (self);
     return;
+  }
 
   if (priv->selected_index < index + removed) {
     /* The item selected item was removed (or none is selected) */
@@ -249,6 +255,8 @@ bound_model_changed (GListModel *list,
     new_idx = 0;
 
   hdy_combo_row_set_selected_index (self, new_idx);
+
+  update (self);
 }
 
 static void
@@ -264,7 +272,8 @@ hdy_combo_row_activate (HdyActionRow *row)
   HdyComboRow *self = HDY_COMBO_ROW (row);
   HdyComboRowPrivate *priv = hdy_combo_row_get_instance_private (self);
 
-  gtk_popover_popup (priv->popover);
+  if (gtk_widget_get_visible (GTK_WIDGET (priv->image)))
+    gtk_popover_popup (priv->popover);
 }
 
 static void
@@ -287,8 +296,6 @@ destroy_model (HdyComboRow *self)
   priv->create_current_widget_func = NULL;
   priv->create_widget_func_data = NULL;
   priv->create_widget_func_data_free_func = NULL;
-
-  gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (self), FALSE);
 }
 
 static void
@@ -418,7 +425,7 @@ hdy_combo_row_class_init (HdyComboRowClass *klass)
    *
    * %TRUE to set the current value as the subtitle.
    *
-   * If you use a custom widget creation function, you will need to give @self
+   * If you use a custom widget creation function, you will need to give the row
    * a name conversion closure with hdy_combo_row_set_get_name_func().
    *
    * If %TRUE, you should not access HdyActionRow:subtitle.
@@ -451,7 +458,7 @@ hdy_combo_row_init (HdyComboRow *self)
 
   priv->selected_index = -1;
 
-  g_signal_connect_object (priv->list, "row-activated", G_CALLBACK (gtk_widget_hide),
+  g_signal_connect_object (priv->list, "row-activated", G_CALLBACK (gtk_popover_popdown),
                            priv->popover, G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->list, "row-activated", G_CALLBACK (row_activated_cb),
                            self, G_CONNECT_SWAPPED);
@@ -563,8 +570,6 @@ hdy_combo_row_bind_model (HdyComboRow                *self,
 
   gtk_list_box_bind_model (priv->list, model, create_list_widget, self, create_list_widget_data_free);
 
-  gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (self), TRUE);
-
   update (self);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED_INDEX]);
